/*
 * snul.vala is a part of Snul
 *
 * Snul: is a simple library that implements
 *       a tcp socket client in vala
 *
 * Copyright (C) 2008  Andrea Del Signore
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 *
 * Author:
 *     Andrea Del Signore <sejerpz@tin.it>
 */

#ifndef __SRC_SNUL_H__
#define __SRC_SNUL_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>

G_BEGIN_DECLS


#define SNUL_TYPE_TCP_CLIENT_SOCKET (snul_tcp_client_socket_get_type ())
#define SNUL_TCP_CLIENT_SOCKET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SNUL_TYPE_TCP_CLIENT_SOCKET, SnulTcpClientSocket))
#define SNUL_TCP_CLIENT_SOCKET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SNUL_TYPE_TCP_CLIENT_SOCKET, SnulTcpClientSocketClass))
#define SNUL_IS_TCP_CLIENT_SOCKET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SNUL_TYPE_TCP_CLIENT_SOCKET))
#define SNUL_IS_TCP_CLIENT_SOCKET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SNUL_TYPE_TCP_CLIENT_SOCKET))
#define SNUL_TCP_CLIENT_SOCKET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), SNUL_TYPE_TCP_CLIENT_SOCKET, SnulTcpClientSocketClass))

typedef struct _SnulTcpClientSocket SnulTcpClientSocket;
typedef struct _SnulTcpClientSocketClass SnulTcpClientSocketClass;
typedef struct _SnulTcpClientSocketPrivate SnulTcpClientSocketPrivate;

typedef enum  {
	SNUL_TCP_SOCKET_ERROR_LOOKUP_SERVICE_FAILED,
	SNUL_TCP_SOCKET_ERROR_LOOKUP_HOSTNAME_FAILED,
	SNUL_TCP_SOCKET_ERROR_SOCKET_CREATION_ERROR,
	SNUL_TCP_SOCKET_ERROR_SOCKET_CONNECT_ERROR,
	SNUL_TCP_SOCKET_ERROR_SOCKET_ALREADY_OPEN,
	SNUL_TCP_SOCKET_ERROR_SOCKET_CLOSED,
	SNUL_TCP_SOCKET_ERROR_SOCKET_BROKEN
} SnulTcpSocketError;
#define SNUL_TCP_SOCKET_ERROR snul_tcp_socket_error_quark ()
typedef enum  {
	SNUL_CHANNEL_ERROR_CODES_ERROR,
	SNUL_CHANNEL_ERROR_CODES_HUP,
	SNUL_CHANNEL_ERROR_CODES_NVAL
} SnulChannelErrorCodes;
struct _SnulTcpClientSocket {
	GObject parent_instance;
	SnulTcpClientSocketPrivate * priv;
};
struct _SnulTcpClientSocketClass {
	GObjectClass parent_class;
};

GQuark snul_tcp_socket_error_quark (void);
gint snul_tcp_client_socket_connect (SnulTcpClientSocket* self, const char* address, const char* port, GError** error);
gsize snul_tcp_client_socket_send (SnulTcpClientSocket* self, const char* buffer);
void snul_tcp_client_socket_disconnect (SnulTcpClientSocket* self);
void snul_tcp_client_socket_on_connected (SnulTcpClientSocket* self, gint socket_fd);
void snul_tcp_client_socket_on_data_received (SnulTcpClientSocket* self, const char* buffer, gsize len);
void snul_tcp_client_socket_on_disconnected (SnulTcpClientSocket* self);
void snul_tcp_client_socket_on_error (SnulTcpClientSocket* self, GError* error);
SnulTcpClientSocket* snul_tcp_client_socket_new (void);
char* snul_tcp_client_socket_get_address (SnulTcpClientSocket* self);
char* snul_tcp_client_socket_get_port (SnulTcpClientSocket* self);
GType snul_tcp_client_socket_get_type (void);

G_END_DECLS

#endif
